package org.lejos.ev3.snake;

import java.util.ArrayList;
import java.util.List;

import lejos.hardware.BrickFinder;
import lejos.hardware.Button;
import lejos.hardware.lcd.Font;
import lejos.hardware.lcd.GraphicsLCD;


/**
 * @author 		Jarema Czajkowski
 * @created 	07-07-2016
 * @license 	The MIT License
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 * 
 * Buttons:
 * ESC		Exit from tool
 * ENTER	No action
 * OTHERS	Move virtual animal
 * 
 */ 
public class Snake {
	static GraphicsLCD device = BrickFinder.getDefault().getGraphicsLCD();
	
	static final int step = 10;
	static final int UIMargin = 2;
	static final int arcSize = 10;
	
	static int width, height;
	
	static int mapW = device.getWidth() - UIMargin*2;
	static int mapH = device.getHeight() - UIMargin*2;

	private static char snakeDirection;
	private static int snakeX ,snakeY ,snakeLength ,foodX ,foodY ,score;

	private static List<int[]> snake = new ArrayList<int[]>();


	public static void main(String[] args) {
		/* VARIABLES INITIALIZATION */
		width = height = snakeX = snakeY = step;
		foodX = foodY = 50;
		
		snakeDirection = 'R';
		snakeLength = 3;
		
		score = 0;
		
		/* INITIATE SNAKE */		
		for (int n=1; n<=snakeLength;n++) {
			int[] init_cord = new int[]{snakeX*n ,snakeY};
			snake.add(init_cord);	
			block(snakeX*n ,snakeY, false);
		}
		snakeX *= snakeLength;
		block(foodX ,foodY, false);
		
		
		for(;;) {
			int buttoncode = Button.waitForAnyPress();
			if (buttoncode == Button.ID_ESCAPE)
		        System.exit(1); 				
			
		/* DIRECTION */
			switch(buttoncode) {
			case Button.ID_UP:
			if (snakeDirection != 'D') {
				snakeDirection = 'U';
			}
			break;
			
			case Button.ID_DOWN:
			if (snakeDirection != 'U') {
				snakeDirection = 'D';
			}
			break;
			
			case Button.ID_LEFT:
			if (snakeDirection != 'R') {
				snakeDirection = 'L';
			}
			break;
			
			case Button.ID_RIGHT:
			if (snakeDirection != 'L') {
				snakeDirection = 'R';
			}
			break;
			}		
			
		/* MOVE SNAKE */
			switch(snakeDirection) {
			case 'U':
			snakeY -= step;
			break;
			
			case 'D':
			snakeY += step;
			break;
			
			case 'L':
			snakeX -= step;
			break;
			
			case 'R':
			snakeX += step;
			}			
			
			
		/* NEW BLOCK */
			int[] cord = new int[]{snakeX ,snakeY};
			
		/* CHECK COLLISION */
			boolean positionOnMap = false;
			for (int i=0;i<snake.size();i++) {
				int[] block = snake.get(i);
				if (block[0]==snakeX && block[1] == snakeY) { positionOnMap = true; break; }
			}
			
		/* PUSH BLOCK */
			snake.add(cord);
			block(snakeX, snakeY, false);			
			
		/* GAME OVER */	
			if (snakeX < 0 || snakeX > mapW || snakeY < 0 || snakeY > mapH || positionOnMap)
			{
				int anchor = GraphicsLCD.BASELINE;
				int x = mapW / 4;
				int y = mapH / 2;
				
				device.clear();
				device.drawString("GAME OVER", x, y, anchor );
				device.setFont(Font.getSmallFont());
				device.drawString(String.valueOf(score*10) + " points", x, y+10, anchor);
				
				Button.waitForAnyPress();
				System.exit(0);
			}				
	
		/* REMOVE TAIL BLOCK */		
			if (snake.size() > snakeLength) {
				int[] el = snake.remove(0);
				block(el[0] ,el[1] ,true);
			}		

			if (snakeX == foodX && snakeY == foodY) {
		/* FEED */
				snakeLength++;
				score++;				
				snake.add(cord);
		/* SPAWN FOOD */
				int availablePositions = mapW * mapH - snakeLength;
				int position = (int) Math.floor(Math.random() * availablePositions/2);
				int realPosition = 0;
				int countdown = position;
				
				while(countdown > 0) {
					countdown--;
					realPosition++;
				}		
				
				foodX = realPosition % mapW;
				foodY = (int) Math.floor(realPosition / mapW);
				
				int modulus = foodX % step;
				if (modulus < step) foodX += step - modulus;
				modulus = foodY % step;
				if (modulus < step) foodY += step - modulus;

			}
			block(foodX ,foodY ,false);

		}
	}
	
	private static void block(int x, int y, boolean white) {
		int color = white ? GraphicsLCD.WHITE : GraphicsLCD.BLACK;
		device.setColor(color);
		device.fillRect(x ,y ,width ,height);		
	}
}
